function [data,beta_interaction] = estimate_parameters(controls,distances)
        % Top level function used in Matlab Application
            
        data = read_excel_data();
        
        reps = 100;
        rng("default")
        warning('off','all')
    
    
        %Initializing variables
        [delta_point,beta_point] = deal(zeros(size(distances)));
        [delta_cf_low,delta_cf_up] = deal(zeros(size(distances)));
        [beta_cf_low,beta_cf_up,kappa,q2] = deal(zeros(size(distances)));
    
    
    
        % Create Progress Bar
        steps = length(distances);
    
        f = waitbar(0,'1','Name','Progress',...
        'CreateCancelBtn','setappdata(gcbf,''canceling'',1)');
    
    
        beta_interaction = 0;
        index = 1;
        for cut_off_dist = distances
    
            waitbar(index/steps,f,sprintf('Estimating Values %d/%d',index,steps ));
            
            if getappdata(f,'canceling')
                delete(f)
                break
            end
    
    
    
    
            [First,Boots,error] =   bootstrap_func(data,cut_off_dist,controls,reps); 
    
            if error == "Error: Distance too large"
                delta_point(index) = NaN;
                delta_cf_up(index) =  NaN;
                delta_cf_low(index) =  NaN;
                
                beta_point(index) = NaN;
                beta_cf_up(index) = NaN;
                beta_cf_low(index) = NaN;
                q2(index) = NaN;
                kappa(index) = NaN;
            else
                delta_point(index) = First.delta(1);
                delta_cf_up(index) =  delta_point(index) + 1.96*std(Boots.delta,0,2,'omitnan');
                delta_cf_low(index) =  delta_point(index) - 1.96*std(Boots.delta,0,2,'omitnan');
                
                beta_point(index) = First.beta(1);
                beta_cf_up(index) = beta_point(index) + 1.96*std(Boots.beta(1,:),0,2,'omitnan');
                beta_cf_low(index) = beta_point(index) - 1.96*std(Boots.beta(1,:),0,2,'omitnan');
                
                q2(index) = First.beta(2);
        
                beta_interaction = First.BetaInt;
        
                kappa(index) = First.kappa;
    
            end
            index = index + 1;  
    
        end

        ident_cond = ((delta_point >= 0 )&(kappa >= (delta_point -1)./delta_point))| ((delta_point < 0 )&(kappa <= (delta_point -1)./delta_point));
    
        data = table(distances',delta_point',delta_cf_low',delta_cf_up',beta_point',beta_cf_low',beta_cf_up',q2',kappa',ident_cond');
        data.Properties.VariableNames = {'tau' 'delta' 'delta_cf_up' 'delta_cf_low' 'beta' 'beta_cf_low' 'beta_cf_up','q2','kappa','ident_cond' };
        
    
        data.beta_bound_low = beta_interaction./data.delta;
        data.beta_bound_up = beta_interaction* (data.delta-1)./data.delta;
       
    
        data.unident_bounded = (isnan(data.beta))  & (data.delta > 2);
        data.unident_unbounded = (data.ident_cond == false) & (data.delta < 0);
        data.unident_amplif = (data.ident_cond == false) & (data.delta > 1) & (data.delta < 2);
        data.unident_atten = (data.ident_cond == false)  & (data.delta > 0) & (data.delta < 1);
        data.identified = (data.unident_bounded == false) & (data.unident_unbounded == false);
    
        delete(f)
     

end