function [omhat,parameters,sigma,R2] = OmegaDistanceEstimator(RegLeft,RegRight,Xn,GG,ZeroIndex,StartX)
% Function to estimate omega using NLS

nObs = size(Xn,1);
Xdev = Xn - mean(Xn);

solve = RegRight*RegLeft;


muHAT = solve(1:nObs);
lambda0= solve(nObs+1:end);
omhat =  GG + muHAT + muHAT' + Xdev*lambda0' + lambda0*Xdev';
omvarLOG = log( diag(omhat).^2);

if sum(isnan(omvarLOG)) > 0
    omhat = 0;
    parameters = 0;
    sigma = 0;
    R2 = 0;
    return
end    

funct = @(param) log((param(1) + Xn).^4) + log(param(2).^4);
StartX = StartX(1:2,:);


SSE = @(param) (omvarLOG - funct(param))'*(omvarLOG - funct(param));



opts = psoptimset('Display','off');
parameters = patternsearch(SSE,StartX,[],[],[],[],[],[],[],opts);

theta = parameters(1);


% Calculate R^2
omvardemean = omvarLOG- mean(omvarLOG);
R2 = 1 -  SSE(parameters)/(omvardemean'*omvardemean);


% Predicted Omega(n,n) from NLLS
omegapredict = exp(funct(parameters)/2);


% Replace values on diagonal
omhat = omhat - diag(diag(omhat))+ diag(omegapredict);


for i = 1:size(ZeroIndex,1)
    m =ZeroIndex(i,1);
    n =ZeroIndex(i,2);
    omhat(m,n) = 0;
    omhat(n,m) = 0;
end




% Calculate Sigma matrix

omtrans = omhat./((Xn+theta)*(Xn+theta)');
d = sqrt(diag(omtrans));
sigma = omtrans./(d*d');




end

