function [First,Boots,error] = bootstrap_func(Data,cut_off_dist,AddControls,reps)
% Function to estimate bootstrapped parameters

switch AddControls
    
    case 0
        
        %Pull Data from NunnStructure
        y = Data.y;
        Xn = Data.Xn;
        ZUSA = Data.ZUSA;
        
        % Calculate interaction Estimator
        betaInt = Interact_Replic(y,Xn,ZUSA);
        
        %Estimate Gamma Matrice
        GG = estimate_gamma(y,Xn,ZUSA);
        
    case 1
        
        %Pull Data from NunnStructure
        
        y = Data.y;
        Xn = Data.Xn;
        ZUSA = Data.ZUSA;
        
        control_1 = Data.control_1;
        control_2 = Data.control_2;
        control_3 = Data.control_3;
        control_4 = Data.control_4;
        

        %Remove Countries that do not have data on Skill and Cap
        [Xn,y,control_1,control_2,control_3,control_4] =remove_empty_countries(Xn,y,control_1,control_2,control_3,control_4);
        
        % Calculate interaction Estimator
        betaInt = Interact_Replic(y,Xn,ZUSA,control_1,control_2,control_3,control_4);
        
        
        %Estimate Gamma Matrice        
        GG = estimate_gamma(y,Xn,ZUSA,control_1,control_2);
        
end


nObs = length(Xn);



[Boots.beta, Boots.sigma,Boots.CoeffsRhos] = deal(nan(2,reps));
Boots.CoeffsRhosSqu = nan(3,reps);


Boots.param = nan(2,reps);



% Index of Diagonals to remove for rho regs
[D1,D2] = find( eye(nObs) == 0);
Bottom = D1>D2;
D1(Bottom) = [];
D2(Bottom) = [];


    
    
% Switch determining whether to use controls in calculations    
switch AddControls
    case 0
        [GG,yhat,uhat] = estimate_gamma(y,Xn,ZUSA);
    case 1
        [GG,yhat,uhat] = estimate_gamma(y,Xn,ZUSA,control_1,control_2);
end




Distance = abs(Xn-Xn')>cut_off_dist; % boolean variable true if x_n - x_m >D

[M,N] = find(Distance == 1);
ZeroIndex = [M N];    

Conditions = createequations_RightSide(Xn,ZeroIndex);

RegLeft = createequations_LeftSide(GG,ZeroIndex);

% Add restriction that sum of lambdas = 0
Conditions(length(ZeroIndex)+1,nObs+1:end) = 1;
RegLeft(end,1)= 0;


% catch warning message
% reset the lastwarn message and id
lastwarn('', '');

RegRight = (Conditions'*Conditions)\Conditions';

% now if a warning was raised, error will not be empty.
error = lastwarn();

StartX = [1 1 1 1 1]';

RemoveIndex = [ZeroIndex; [D1 D2];[(1:nObs)' (1:nObs)']];

[omegahat,param,sigma] = OmegaDistanceEstimator(RegLeft,RegRight,Xn,GG,ZeroIndex,StartX);

%Check for errors
if omegahat == 0
    First = 0;
    Boots = 0;
    error = "Error: Distance too large";
    return
else
    First  = SaveFirst(param,betaInt,omegahat,sigma,Xn,ZUSA,RemoveIndex,ZeroIndex);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% START BOOTSTRAP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


for repetition = 1:reps

    uboot =  permutewithreplacement(uhat);
    yboot = yhat + uboot;


    GGboot = estimate_gamma(yboot,Xn,ZUSA);

    betaIntBoot = Interact_Replic(yboot,Xn,ZUSA);


    NewRegLeft = createequations_LeftSide(GGboot,ZeroIndex);
    RegLeft(1:end-1,1) = NewRegLeft(1:end-1,1);


    [omegaboot,paramBOOT,sigmaboot] = OmegaDistanceEstimator(RegLeft,RegRight,Xn,GGboot,ZeroIndex,StartX);


    Boots  = SaveBoots(Boots,paramBOOT,betaIntBoot,omegaboot,sigmaboot,Xn,ZUSA,RemoveIndex,repetition);


end



end

